var chan = async_load[?"channel_index"];
var buff = async_load[?"buffer_id"];
var size = async_load[?"data_len"];

if !(global.connected)
	{
	if (record_channel > -1)
		{
		audio_stop_recording(record_channel);
		record_channel = -1;
		}
	exit;
	}

if (chan == record_channel)
	{
	var audio_buffer = buffer_create(size,buffer_fixed,1);
	buffer_copy(buff,0,size,audio_buffer,0);
	
	if (device_format == buffer_s16) // 16bit record + 8bit transmit
		{
		var _add = 32768;
		var _div = 65535;
		var skip = 2;
		}
	else if (device_format == buffer_u8) // 8bit record + 8bit transmit
		{
		var _add = 0;
		var _div = 255;
		var skip = 1;
		}
	
	var amp = 0;
	for(var i=0; i<size; i+=skip;)
		amp = max(amp,(_add+buffer_peek(audio_buffer,i,device_format))/_div);
	amp = bias(amp,0.75);
	ds_map_set(playback_info[user_id],"amp",amp);
	
	#region audio_parameters
		
		encode_format = buffer_u8;
		encode_sample = device_sample;
		encode_bytes = size;
		write_skip = 1;
		read_skip = 1;
		
		// only transmit 16bits if we can record 16bits
		if (device_format == buffer_s16)
			{
			read_skip *= 2;
			if (transmit_format == buffer_s16)
				{
				encode_format = buffer_s16;
				write_skip *= 2;
				}
			}
		
		if (device_stereo == audio_stereo) // downsample to mono
			{
			encode_sample /= 2;
			encode_bytes /= 2;
			read_skip *= 2;
			}
		
		if (transmit_downsample) // downsample
			{
			encode_sample /= 2;
			encode_bytes /= 2;
			read_skip *= 2;
			}
		
		if (encode_format == buffer_s16) // 16bit record + 16bit transmit
			{
			_add = 0;
			_div = 1;
			}
		else
			{
			if (device_format == buffer_s16) // 16bit record + 8bit transmit
				{
				encode_bytes /= 2;
				_add = 32768;
				_div = 256;
				}
			else if (device_format == buffer_u8) // 8bit record + 8bit transmit
				{
				_add = 0;
				_div = 1;
				}
			}
	
	show_debug_message("DEVICE STEREO: "+string(device_stereo));
	show_debug_message("DEVICE FORMAT: "+string(device_format));
	show_debug_message("TRANSMIT FORMAT: "+string(transmit_format));
	show_debug_message("ENCODE FORMAT: "+string(encode_format));
	show_debug_message("TRANSMIT DOWNSAMPLE: "+string(transmit_downsample));
	show_debug_message("DEVICE SAMPLE: "+string(device_sample));
	show_debug_message("ENCODE SAMPLE: "+string(encode_sample));
	//show_debug_message("_ADD: "+string(_add));
	//show_debug_message("_DIV: "+string(_div));
	show_debug_message("DEVICE BYTES: "+string(size));
	show_debug_message("ENCODE BYTES: "+string(ceil(encode_bytes)));
	
	#endregion audio_parameters
	
	// if the audio signal is above our noise gate threshold
	if (amp >= gate)
		{
		bytes_read = 0;
		while(bytes_read < size)
			{
			var temp_size = min(size,bytes_read+(MTU-128));
			var sample_buffer = buffer_create((MTU-128),buffer_fixed,1);
			
			// copy relevant audio data from record buffer to transmit buffer
			var offset = 0;
			for(var i=bytes_read; i<temp_size; i+=read_skip;)
				{
				val = (_add+buffer_peek(audio_buffer,i,device_format))/_div;
				buffer_poke(sample_buffer,offset,transmit_format,round(val*gain));
				offset += write_skip;
				}
			bytes_read = temp_size;
			
			// compress the buffer and get its size
			var comp_buffer = buffer_compress(sample_buffer,0,offset);
			if (comp_buffer >= 0)
				{
				var comp_bytes = buffer_get_size(comp_buffer);
				show_debug_message("TRANSMIT BYTES: "+string(comp_bytes));
				
				// send out the buffer
				with(obj_client)
					{
					var send_buffer = buffer_create(16+comp_bytes,buffer_fixed,1);
					buffer_seek(send_buffer,buffer_seek_start,0);
					buffer_write(send_buffer,buffer_u8,user_message.user_voice);
					buffer_write(send_buffer,buffer_u8,true);
					buffer_write(send_buffer,buffer_u8,other.encode_format);
					buffer_write(send_buffer,buffer_u16,other.encode_sample);
					buffer_write(send_buffer,buffer_u16,comp_bytes);
					
					var pos = buffer_tell(send_buffer);
					buffer_copy(comp_buffer,0,comp_bytes,send_buffer,pos);
					
					user_queue_message(send_buffer,pos+comp_bytes);
					}
				
				buffer_delete(comp_buffer);
				}
			else
				log("BUFFER COMPRESSION FAILED: "+string(comp_buffer));
			
			buffer_delete(sample_buffer);
			}
		}
	
	buffer_delete(audio_buffer);
	}

if !(transmit)
	{
	if (record_channel >= 0)
		{
		ds_map_set(playback_info[user_id],"amp",0);
		audio_stop_recording(record_channel);
		record_channel = -1;
		
		show_debug_message("OBJ_CHAT_CONTROL: RECORDING STOPPED");
		
		// send out the buffer
		with(obj_client)
			{
			var send_buffer = buffer_create(16,buffer_fixed,1);
			buffer_seek(send_buffer,buffer_seek_start,0);
			buffer_write(send_buffer,buffer_u8,user_message.user_voice);
			buffer_write(send_buffer,buffer_u8,false);
			user_queue_message(send_buffer);
			}
		}
	exit;
	}