/// user_handle_packet();

buffer_seek(buffer,buffer_seek_start,0);
_messages = buffer_read(buffer,buffer_u8);

repeat(_messages)
	{
	_message_id = buffer_read(buffer,buffer_u8);
	if (_message_id < server_message.size)
		show_debug_message(global.server_message_text[_message_id]);
	
	switch(_message_id)
		{
		case server_message.welcome:
			user_id  = buffer_read(buffer,buffer_u8);
			max_user = buffer_read(buffer,buffer_u8);
			server_name = buffer_read(buffer,buffer_string);
			server_bits = buffer_read(buffer,buffer_u8);
			server_down = buffer_read(buffer,buffer_u8);
			cur_user = 0;
			
			// setup user info
			for(var i=0; i<max_user; i++;)
				{
				connected[i] = false;
				user_info[i] = ds_map_create();
				}
			
			with(obj_chat_control)
				{
				if (other.server_bits == 0)
					transmit_format = buffer_u8;
				else if (other.server_bits == 1)
					transmit_format = buffer_s16;
				if (other.server_down == 0)
					transmit_downsample = false;
				else if (other.server_down == 1)
					transmit_downsample = true;
				}
			
			log("Connected to "+string(server_name)+"!");
			with(obj_chat_control)
				{
				chat_time = get_timer();
				ds_list_insert(chat_list,0,"Connected to "+string(other.server_name)+"!");
				}
			
			global.connected = true;
			break;
		
		case server_message.goodbye:
			global.connected = false;
			
			log("Server shutdown! Disconnected from "+string(server_name)+"!");
			with(obj_chat_control)
				{
				chat_time = get_timer();
				ds_list_insert(chat_list,0,"Server shutdown! Disconnected from "+string(other.server_name)+"!");
				}
			
			for(var i=0; i<messages; i++;)
				{
				var buff = message_list[|i*2];
				buffer_delete(buff);
				}
			ds_list_destroy(message_list);
			messages = 0;
			
			for(var i=0; i<max_user; i++;)
				{
				if !(connected[i])
					continue;
				
				var temp_info = user_info[i];
				if (ds_exists(temp_info[?"img_data"],ds_type_grid))
					ds_grid_destroy(temp_info[?"img_data"]);
				if (sprite_exists(temp_info[?"img"]))
					sprite_delete(temp_info[?"img"]);
				ds_map_destroy(user_info[i]);
				
				connected[i] = false;
				}
			
			network_destroy(socket);
			
			cur_user = 0;
			max_user = 0;
			server_name = "";
			break;
		
		case server_message.user_join:
			var num = buffer_read(buffer,buffer_u8);
			repeat(num)
				{
				var temp_id		= buffer_read(buffer,buffer_u8);
				var temp_name	= buffer_read(buffer,buffer_string);
				
				connected[temp_id] = true;
				ds_map_set(user_info[temp_id],"name",temp_name);
				ds_map_set(user_info[temp_id],"ping",0);
				
				var temp_data = ds_grid_create(64,64);
				ds_map_set(user_info[temp_id],"img_data",temp_data);
				ds_map_set(user_info[temp_id],"img_sectors",0);
				ds_map_set(user_info[temp_id],"img_ready",false);
				ds_map_set(user_info[temp_id],"img",-1);
				}
			cur_user += num;
			
			global.user_update_callback = true;
			var aud = audio_play_sound(snd_connect,0,0);
			audio_sound_gain(aud,0.8,0);
			break;
		
		case server_message.user_leave:
			var num = buffer_read(buffer,buffer_u8);
			repeat(num)
				{
				var temp_id	= buffer_read(buffer,buffer_u8);
				
				connected[temp_id] = false;
				
				ds_grid_destroy(ds_map_find_value(user_info[temp_id],"img_data"));
				if (sprite_exists(ds_map_find_value(user_info[temp_id],"img")))
					sprite_delete(ds_map_find_value(user_info[temp_id],"img"));
				ds_map_clear(user_info[temp_id]);
				}
			cur_user -= num;
			
			global.user_update_callback = true;
			var aud = audio_play_sound(snd_disconnect,0,0);
			audio_sound_gain(aud,0.8,0);
			break;
		
		case server_message.user_image_upload:
			img_upload_sectors = buffer_read(buffer,buffer_u16);
			if (img_upload_sectors == 65535)
				img_uploaded = true;
			break;
		
		case server_message.user_image_download:
			var temp_id = buffer_read(buffer,buffer_u8);
			var sector = buffer_read(buffer,buffer_u8);
			if (connected[temp_id]) and (temp_id == img_download)
				{
				var temp_data = ds_map_find_value(user_info[temp_id],"img_data");
				var sectors = ds_map_find_value(user_info[temp_id],"img_sectors");
				for(var iy=sector*4; iy<sector*4+4; iy++;)
					{
					for(var ix=0; ix<64; ix++;)
						ds_grid_set(temp_data,ix,iy,buffer_read(buffer,buffer_u16));
					}
				
				sectors = sectors | (1 << sector);
				ds_map_set(user_info[temp_id],"img_sectors",sectors);
				
				if (sectors == 65535)
					{
					ds_map_set(user_info[temp_id],"img_ready",true);
					img_download = -1;
					}
				}
			else
				buffer_seek(buffer,buffer_seek_start,buffer_tell(buffer)+512);
			break;
		
		case server_message.user_message:
			var temp_id = buffer_read(buffer,buffer_u8);
			var chat = buffer_read(buffer,buffer_string);
			with(obj_chat_control)
				{
				chat_time = get_timer();
				ds_list_insert(chat_list,0,string(ds_map_find_value(playback_info[temp_id],"name"))+": "+string(chat));
				}
			
			var aud = audio_play_sound(snd_message,0,0);
			audio_sound_gain(aud,0.8,0);
			break;
		
		case server_message.user_voice:
			var temp_id = buffer_read(buffer,buffer_u8);
			var transmit = buffer_read(buffer,buffer_u8);
			if (transmit)
				{
				// user is transmitting audio
				var format = buffer_read(buffer,buffer_u8);
				var sample = buffer_read(buffer,buffer_u16);
				var bytes = buffer_read(buffer,buffer_u16);
				
				var pos = buffer_tell(buffer);
				
				var temp_buffer = buffer_create(bytes,buffer_fixed,1);
				buffer_copy(buffer,pos,bytes,temp_buffer,0);
				
				with(obj_chat_control)
					{
					if (playback)
						{
						ds_queue_enqueue(zipped_queue[temp_id],temp_buffer);
						playback_format[temp_id] = format;
						playback_sample[temp_id] = sample;
						}
					else
						buffer_delete(temp_buffer);
					}
				
				// seek to where we left off reading in the async buffer
				buffer_seek(buffer,buffer_seek_start,pos+bytes);
				}
			else
				{
				// user has stopped transmitting audio
				playback[temp_id] = false;
				}
			break;
		
		case server_message.heartbeat:
			var broadcast = buffer_read(buffer,buffer_u8);
			if (broadcast)
				{
				var num = buffer_read(buffer,buffer_u8);
				repeat(num)
					{
					var temp_id = buffer_read(buffer,buffer_u8);
					var temp_ping = buffer_read(buffer,buffer_u16);
					
					ds_map_set(user_info[temp_id],"ping",temp_ping*16);
					}
				}
			
			var temp_buff = buffer_create(16,buffer_fixed,1);
			buffer_seek(temp_buff,buffer_seek_start,0);
			buffer_write(temp_buff,buffer_u8,user_message.heartbeat);
			user_queue_message(temp_buff);
			break;
		
		case server_message.error:
			log("ERROR: Failed to connect to server!");
			
			network_destroy(socket);
			break;
		}
	}